
function dcl.InDimension()
	-- this function checks if the player is currently in a Dimension, returns true or false
	if dcl.Force == true then return true end;
	if Inspect.Unit.Detail("player").locationName ~= nil then
		local location = Inspect.Unit.Detail("player").locationName;
		if string.find(location, dcl.GetString(2)) then return true else return false end;
	end
	return false;
end


function dcl.TraceDimParameters()
	-- This function tracks player coords around the outside of the Dimension
	-- Should return the heighest and lowest values for x, y and z.
	dcl.DimBoundMin.X = dcl.Round(math.min(dcl.DimBoundMin.X,dcl.PlayerCoords.X),4);
	dcl.DimBoundMax.X = dcl.Round(math.max(dcl.DimBoundMax.X,dcl.PlayerCoords.X),4);
	dcl.DimBoundMin.Y = dcl.Round(math.min(dcl.DimBoundMin.Y,dcl.PlayerCoords.Y),4);
	dcl.DimBoundMax.Y = dcl.Round(math.max(dcl.DimBoundMax.Y,dcl.PlayerCoords.Y),4);
	dcl.DimBoundMin.Z = dcl.Round(math.min(dcl.DimBoundMin.Z,dcl.PlayerCoords.Z),4);
	dcl.DimBoundMax.Z = dcl.Round(math.max(dcl.DimBoundMax.Z,dcl.PlayerCoords.Z),4);
end


function dcl.GetDimCentre()
	local centre = {["X"]=0;["Y"]=0;["Z"]=0;};
	centre.X = dcl.Round(((dcl.DimBoundMax.X-dcl.DimBoundMin.X)/2)+dcl.DimBoundMin.X,4);
	centre.Y = dcl.Round(((dcl.DimBoundMax.Y-dcl.DimBoundMin.Y)/2)+dcl.DimBoundMin.Y,4);
	centre.Z = dcl.Round(((dcl.DimBoundMax.Z-dcl.DimBoundMin.Z)/2)+dcl.DimBoundMin.Z,4);
	return centre;
end


function dcl.GetDimDimensions()
	local dimensions = {["X"]=0;["Y"]=0;["Z"]=0;};
	dimensions.X = dcl.Round(dcl.DimBoundMax.X-dcl.DimBoundMin.X,4);
	dimensions.Y = dcl.Round(dcl.DimBoundMax.Y-dcl.DimBoundMin.Y,4);
	dimensions.Z = dcl.Round(dcl.DimBoundMax.Z-dcl.DimBoundMin.Z,4);
	return dimensions;
end


function dcl.UpdateCoordsDisplay()
	dcl.lblCoordsX:SetText("X: "..dcl.PlayerCoords.X);
	dcl.lblCoordsY:SetText("Y: "..dcl.PlayerCoords.Y);
	dcl.lblCoordsZ:SetText("Z: "..dcl.PlayerCoords.Z);

	dcl.lblCoordsMinX:SetText("X: "..dcl.DimBoundMin.X);
	dcl.lblCoordsMinY:SetText("Y: "..dcl.DimBoundMin.Y);
	dcl.lblCoordsMinZ:SetText("Z: "..dcl.DimBoundMin.Z);

	dcl.lblCoordsMaxX:SetText("X: "..dcl.DimBoundMax.X);
	dcl.lblCoordsMaxY:SetText("Y: "..dcl.DimBoundMax.Y);
	dcl.lblCoordsMaxZ:SetText("Z: "..dcl.DimBoundMax.Z);

	local dimCentre = dcl.GetDimCentre();
	dcl.lblCoordsCentreX:SetText("X: "..dimCentre.X);
	dcl.lblCoordsCentreY:SetText("Y: "..dimCentre.Y);
	dcl.lblCoordsCentreZ:SetText("Z: "..dimCentre.Z);

	local dimDims = dcl.GetDimDimensions();
	dcl.lblBoundDimsX:SetText("X: "..dimDims.X);
	dcl.lblBoundDimsY:SetText("Y: "..dimDims.Y);
	dcl.lblBoundDimsZ:SetText("Z: "..dimDims.Z);
end


function dcl.ResetCoords()
	dcl.PlayerCoords.X = dcl.Round(Inspect.Unit.Detail("player").coordX,4);;
	dcl.PlayerCoords.Y = dcl.Round(Inspect.Unit.Detail("player").coordY,4);;
	dcl.PlayerCoords.Z = dcl.Round(Inspect.Unit.Detail("player").coordZ,4);;
	dcl.DimBoundMin.X = dcl.PlayerCoords.X;
	dcl.DimBoundMin.Y = dcl.PlayerCoords.Y;
	dcl.DimBoundMin.Z = dcl.PlayerCoords.Z;
	dcl.DimBoundMax.X = dcl.PlayerCoords.X;
	dcl.DimBoundMax.Y = dcl.PlayerCoords.Y;
	dcl.DimBoundMax.Z = dcl.PlayerCoords.Z;
	dcl.UpdateCoordsDisplay();
end


function dcl.GetItemscentre(listItems)
	-- Returns information about a group of items and their position boundaries
	if type(listItems) ~= 'table' then return end;

	local groupMin = {};
	local groupMax = {};

	groupMin.X = listItems[1].coordX;
	groupMin.Y = listItems[1].coordY;
	groupMin.Z = listItems[1].coordZ;
	groupMax.X = listItems[1].coordX;
	groupMax.Y = listItems[1].coordY;
	groupMax.Z = listItems[1].coordZ;

	for k,v in pairs(listItems) do
		groupMin.X = math.min(groupMin.X,v.coordX);
		groupMin.Y = math.min(groupMin.Y,v.coordY);
		groupMin.Z = math.min(groupMin.Z,v.coordZ);
		groupMax.X = math.max(groupMax.X,v.coordX);
		groupMax.Y = math.max(groupMax.Y,v.coordY);
		groupMax.Z = math.max(groupMax.Z,v.coordZ);
	end

	local centre = {["X"]=0;["Y"]=0;["Z"]=0;};
	centre.X = ((groupMax.X-groupMin.X)/2)+groupMin.X;
	centre.Y = ((groupMax.Y-groupMin.Y)/2)+groupMin.Y;
	centre.Z = ((groupMax.Z-groupMin.Z)/2)+groupMin.Z;

	return centre,groupMin,groupMax;
end


function dcl.GetSnapshot()

	local itemList = Inspect.Dimension.Layout.List();
	local playerInfo = Inspect.Unit.Detail("player");
	local perimeter = dcl.StringToNumber(dcl.txtRadius:GetText(),150,dcl.GetString(68));
	if perimeter == nil then return end;
	local tSnapshot = {};
	local skippedCount = 0;
	for k,v in pairs (itemList) do
		if v == false then
			local item = Inspect.Dimension.Layout.Detail(k);
			if item ~= nil and dcl.InPerimeter(item.coordX,item.coordY,item.coordZ,playerInfo.coordX,playerInfo.coordY,playerInfo.coordZ,perimeter) then
				table.insert(tSnapshot,{["name"]=item.name;["id"]=k;["type"]=item.type;["coordX"]=item.coordX;["coordY"]=item.coordY;["coordZ"]=item.coordZ;["pitch"]=item.pitch;["roll"]=item.roll;["yaw"]=item.yaw;["scale"]=item.scale;});
			else
				skippedCount = skippedCount + 1;
			end
		end
	end
	return tSnapshot,skippedCount;
end


function dcl.GetDimensionSize()
	local items = 0;
	local itemList = Inspect.Dimension.Layout.List();
	for k,v in pairs (itemList) do
		if v == false then items = items + 1 end;
	end
	return items;
end


function dcl.IsInDimension(itemInfo)
	-- This function checks an item during load/restore to make sure it doesn't already exist in the dimension
	-- itemInfo is the table details about the item being loaded. Returns true or false.
	if itemInfo == nil or type(itemInfo) ~= 'table' then return false end;
	local itemList = Inspect.Dimension.Layout.List();
	for k,v in pairs (itemList) do
		if v==false then
			local item = Inspect.Dimension.Layout.Detail(k);
			if item ~= nil then
				if item.type==itemInfo.type and dcl.abs(item.coordX)==dcl.abs(itemInfo.coordX) and dcl.abs(item.coordY)==dcl.abs(itemInfo.coordY) and dcl.abs(item.coordZ)==dcl.abs(itemInfo.coordZ) and dcl.abs(item.roll)==dcl.abs(itemInfo.roll) and dcl.abs(item.pitch)==dcl.abs(itemInfo.pitch) and dcl.abs(item.yaw)==dcl.abs(itemInfo.yaw) and dcl.abs(item.scale)==dcl.abs(itemInfo.scale) then
					return true;
				end
			end
		end
	end
	return false;
end


function dcl.UpdateSnapshotID(itemList)
	for k,v in pairs (itemList) do
		local keyID = dcl.GetItemKey(k,dcl.Snapshot);
		if keyID ~= nil then
			dcl.Snapshot[keyID].id = k;
		end
	end
end


function dcl.GetItemKey(itemID,matchTable)
	-- This function matches a dimension item to a table entry in the given table, returning the key - or nil if it doesn't exist.
	if itemID == nil or type(matchTable) ~= 'table' then return end;
	local item = Inspect.Dimension.Layout.Detail(itemID);
	if item ~= nil then
		for k,v in pairs (matchTable) do
			--[[
			if item.type==v.type then
				print(item.name..": "..dcl.abs(item.coordX).." : "..dcl.abs(v.coordX));
				print(item.name..": "..dcl.abs(item.coordY).." : "..dcl.abs(v.coordY));
				print(item.name..": "..dcl.abs(item.coordZ).." : "..dcl.abs(v.coordZ));
				print(item.name..": "..dcl.abs(item.roll).." : "..dcl.abs(v.roll));
				print(item.name..": "..dcl.abs(item.pitch).." : "..dcl.abs(v.pitch));
				print(item.name..": "..dcl.abs(item.yaw).." : "..dcl.abs(v.yaw));
				print(item.name..": "..dcl.abs(item.scale).." : "..dcl.abs(v.scale));
			end --]]
			if item.type==v.type and dcl.abs(item.coordX)==dcl.abs(v.coordX) and dcl.abs(item.coordY)==dcl.abs(v.coordY) and dcl.abs(item.coordZ)==dcl.abs(v.coordZ) and dcl.abs(item.roll)==dcl.abs(v.roll) and dcl.abs(item.pitch)==dcl.abs(v.pitch) and dcl.abs(item.yaw)==dcl.abs(v.yaw) and dcl.abs(item.scale)==dcl.abs(v.scale) then
				return k;
			end
		end
	end
end


function dcl.InPerimeter(itemX,itemY,itemZ,playerX,playerY,playerZ,radius)
	if itemX==nil or itemY==nil or itemZ==nil or playerX==nil or playerY==nil or playerZ == nil then return false end;
	if radius == nil or radius == 0 then radius = 100 end;
	if (itemX-playerX)^2 + (itemY-playerY)^2 + (itemZ-playerZ)^2 > radius^2 then return false end;
	return true;
end


function dcl.ItemStatsChanged(itemInfo)
	local snapID = dcl.GetSnapshotID(itemInfo);
	if snapID ~= nil then
		local snapItem = dcl.Snapshot[snapID];
		if dcl.abs(itemInfo.coordX)~=dcl.abs(snapItem.coordX) or dcl.abs(itemInfo.coordY)~=dcl.abs(snapItem.coordY) or dcl.abs(itemInfo.coordZ)~=dcl.abs(snapItem.coordZ) or dcl.abs(itemInfo.roll)~=dcl.abs(snapItem.roll) or dcl.abs(itemInfo.pitch)~=dcl.abs(snapItem.pitch) or dcl.abs(itemInfo.yaw)~=dcl.abs(snapItem.yaw) or dcl.abs(itemInfo.scale)~=dcl.abs(snapItem.scale) then
			return true;
		end
	end
	return false;
end


function dcl.GetSnapshotID(itemInfo)
	-- This function checks if an item exists already in the current snapshot, to prevent re-adding.
	if type(dcl.Snapshot)~='table' then return false end;
	for k,v in pairs (dcl.Snapshot) do
		if v.id == itemInfo.id then return k end;
	end
end

--[[
function dcl.GetInvSpaceRequirements()
	-- This function loops through dimension items to get the number of inventory 'slots' needed to pack every item
	local slotsNeeded = 0;
	local tItemTypes = dcl.GetDimensionItemList();
	for k,v in pairs (tItemTypes) do
		slotsNeeded = slotsNeeded + math.ceil(v/Inspect.Item.Detail(k).stackMax);
	end
	return slotsNeeded;
end
--]]

function dcl.PrintDimensionItems()
	local tItemTypes = dcl.GetDimensionItemList();
	for k,v in pairs (tItemTypes) do
		local item = Inspect.Item.Detail(k);
		print(tagText(item.name.." = "..v,"LightSlateGray"));
	end
end


function dcl.GetDimensionItemList(reqTable)
	-- This function loops through the dimension items saved in the snapshot, and returns a table list of item types and their quantities.
	if reqTable == nil then reqTable = dcl.Snapshot end;
	local tItemTypes = {};
	for k,v in pairs (reqTable) do
		if tItemTypes[v.type] == nil then
			tItemTypes[v.type] = 1;
		else
			tItemTypes[v.type] = tItemTypes[v.type] + 1;
		end
	end
	return tItemTypes;
end


function dcl.GetItemAvailability(itemType)
	-- This function checks each of the selected sources to get the number of pieces available for the item type.
	-- returns a table with quantities at each source to the callback function
	if itemType == nil then return end;
	local qtyAvail = {};
	qtyAvail["Crate"] = dcl.GetCrateAvailability(itemType);
	qtyAvail["Bags"] = dcl.GetBagAvailability(itemType);
	qtyAvail["Bank"] = dcl.GetBankAvailability(itemType);
	qtyAvail["Vault"] = dcl.GetVaultAvailability(itemType);
	qtyAvail["Guild"] = dcl.GetGuildAvailability(itemType);
	return qtyAvail;
end


function dcl.GetCrateAvailability(itemType)
	local quantity = 0;
	local dimList = Inspect.Dimension.Layout.List();
	for k,v in pairs(dimList) do
		if v == true then
			local item = Inspect.Dimension.Layout.Details(k);
			if item.type == itemType then quantity = quantity + 1 end;
		end
	end
	return quantity;
end


function dcl.GetBagAvailability(itemType)
	local quantity = 0;
	local items = Inspect.Item.Detail(Inspect.Item.List(Utility.Item.Slot.Inventory()));
	for k,v in pairs(items) do
		if v.type == itemType then quantity = quantity + v.stack end;
	end
	return quantity;
end


function dcl.GetBankAvailability(itemType)
	local quantity = 0;
	local items = Inspect.Item.Detail(Inspect.Item.List(Utility.Item.Slot.Bank()));
	for k,v in pairs(items) do
		if v.type == itemType then quantity = quantity + v.stack end;
	end
	return quantity;
end


function dcl.GetVaultAvailability(itemType)
	local quantity = 0;
	local items = Inspect.Item.Detail(Inspect.Item.List(Utility.Item.Slot.Vault()));
	for k,v in pairs(items) do
		if v.type == itemType then quantity = quantity + v.stack end;
	end
	return quantity;
end


function dcl.GetGuildAvailability(itemType)
	local quantity = 0;
	local items = Inspect.Item.Detail(Inspect.Item.List(Utility.Item.Slot.Guild()));
	for k,v in pairs(items) do
		if v.type == itemType then quantity = quantity + v.stack end;
	end
	return quantity;
end


function dcl.GetSlotsNeeded(playerInv,itemType,qty)
	-- This function checks the number of additional slots needed per item to pack the quantity given.
	if playerInv == nil or itemType == nil or qty == nil then return 0 end;
	local stackCount,stackQty = 0,0;
	local stackSize = Inspect.Item.Detail(itemType).stackMax;

	if playerInv[itemType] ~= nil then
		for k,v in pairs(playerInv[itemType]) do
			stackCount = stackCount + 1;
			stackQty = stackQty + v;
		end
	end

	return (math.ceil((stackQty+qty)/stackSize)-stackCount);
end


function dcl.GetPlayerInventory()
	local tInv = {};
	local items = Inspect.Item.List(Utility.Item.Slot.Inventory());
	for k,v in pairs(items) do
		if v ~= false then
			local item = Inspect.Item.Detail(v);
			local itemType = item.type;
			local stack = 1;
			if item.stack ~= nil then stack = item.stack end;
			if tInv[itemType] == nil then tInv[itemType] = {} end;
			table.insert(tInv[itemType],stack);
		end
	end
	return tInv;
end


function dcl.GetInvFreeSpace()
	local freeSlots = 0;
	local capacity = 0;
	local bagMatrix = dcl.GetBagMatrix();
	for k,v in pairs (bagMatrix) do
		for i=1, v do
			local slotID = Utility.Item.Slot.Inventory(k, i);
			local item = Inspect.Item.Detail(slotID);
			if item == nil then freeSlots = freeSlots + 1 end;
		end
		capacity = capacity + v;
	end
	return freeSlots,capacity;
end


function dcl.GetInvCapacity()
	local capacity = 0;
	local bagMatrix = dcl.GetBagMatrix();
	for k,v in pairs (bagMatrix) do
		capacity = capacity + v;
	end
	return capacity;
end


function dcl.GetFreeSlot()
	local bagMatrix = dcl.GetBagMatrix();
	for k,v in pairs (bagMatrix) do
		for i=1, v do
			local slotID = Utility.Item.Slot.Inventory(k, i);
			local item = Inspect.Item.Detail(slotID);
			if item == nil then return slotID end;
		end
	end
end


function dcl.GetBagMatrix()
	local bagMatrix = {};
	for i=1,10 do	-- only seven bag slots at time of release, but allows for additional slots without rebuilding
		local bagID = Utility.Item.Slot.Inventory("bag",i);
		local bag = Inspect.Item.Detail(bagID);
		if bag ~= nil then
			bagMatrix[i] = bag.slots;
		end
	end
	return bagMatrix; -- eg. 1=28; 2=28; 3=20
end


function dcl.Reset()
	if Inspect.Unit.Detail("player").locationName ~= nil then
		dcl.Zone = string.gsub(Inspect.Unit.Detail("player").locationName,dcl.GetString(1),"");
	else
		dcl.Zone = "";
	end
	dcl.txtDimName:SetText(dcl.Zone);
	dcl.TraceCoords = dcl.InDimension();
	dcl.ResetCoords();
	dcl.RefreshListSnapshots();
	dcl.Restore_Co = nil;
	dcl.Update_Co = nil;
	dcl.Remove_Co = nil;
	dcl.Print_Co = nil;
	dcl.ResetSnapshot();
	if dcl.InDimension() then
		dcl.btnSnapshot:SetEnabled(true);
	else
		dcl.btnSnapshot:SetEnabled(false);
	end
end


function dcl.ResetSnapshot()
	dcl.Snapshot = {};
	dcl.btnSnapAdd:SetEnabled(false);
	dcl.btnPrint:SetEnabled(false);
	dcl.btnCrate:SetEnabled(false);
	dcl.btnSave:SetEnabled(false);
	dcl.btnExport:SetEnabled(false);
	dcl.btnRestore:SetEnabled(false);
end

