
function dcl.NewLabel(name,parent,width,height,left,top,text)

	if name == nil or parent == nil then return end;
	if width == nil then width = 100 end;
	if height == nil then height = 18 end;
	if left == nil then left = 0 end;
	if top == nil then top = 0 end;
	if text == nil then text = "" end;

	local label = UI.CreateFrame("Text", name, parent);
	label:SetWidth(width);
	label:SetHeight(height);
	label:SetPoint("TOPLEFT",parent,"TOPLEFT",left,top);
	label:SetFontSize(12);
	label:SetText(text);

	return label;

end


function dcl.GetLanguage()
	local languages = {
		["English"] = 1;
		["French"] = 2;
		["German"] = 3;
		["Korean"] = 4;
		["Russian"] = 5;
		["Chinese"] = 6;
		["Taiwanese"] = 7;
		};

	local lang = Inspect.System.Language();
	if languages[lang] then return languages[lang] end;
	return 1;
end


function dcl.GetString(stringID,langID) -- langID not essential, will take loaded value if missing.
	if stringID == nil then return "" end;
	if langID == nil then langID = dcl.LangID end;
	-- this function checks for the string id in the prefered language
	-- if it can't find in the prefered language then it will search in the default language (English)
	-- if it can't be found still, then string error is returned.
	if dcl.Strings[langID] ~= nil and dcl.Strings[langID][stringID] ~= nil then return dcl.Strings[langID][stringID] end;
	if dcl.Strings[1] ~= nil and dcl.Strings[1][stringID] ~= nil then return dcl.Strings[1][stringID] end;
	return "<string error id:" .. stringID .. ">";
end


function print (text)
	if text == nil then return end;
	text = tostring(text);
	Command.Console.Display("general", true, text, true);
end


function tagText(text,color)
	if text==nil then return "" end;
	if colors[color] == nil then
		return "<font color=\"#FFFFFF\">"..text.."</font>";
	else
		return "<font color=\""..colors[color].."\">"..text.."</font>";
	end
end


function dcl.abs(value)
	local num = tostring(value);
	local retnum = "";
	local workingNum = string.gsub(num,"-","");
	if string.find(workingNum,"%.") == nil then
		retnum = workingNum;
	else
		local sPref,sSuf = string.match(workingNum,"(%d+).(%d+)");
		if sSuf == nil then
			retnum = sPref;
		else
			retnum = sPref.."."..string.sub(sSuf,1,3);
		end
	end
	return tonumber(retnum);
end


-- This function checks data against a specific data type. If it matches
-- then the data is returned, if not then optional error message is printed
-- and optional return value is returned, or nil.
function dcl.ValidateType(data,dataType,errorMessage,errorReturn)
	if data == nil or dataType == nil then return end;
	if type(data) ~= string.lower(dataType) then
		if errorMessage ~= nil then print(errorMessage) end;
		if errorReturn ~= nil then return errorReturn end;
	else
		return data;
	end
end


function dcl.StringToNumber(data,defEmpty,errorMessage,errorReturn)
	if data == nil then return end;
	if type(data) == 'string' then
		if data == "" and defEmpty ~= nil then data = defEmpty end;
		if not tonumber(data) then
			if errorMessage ~= nil then print(errorMessage) end;
			if errorReturn ~= nil then return errorReturn end;
		else
			return tonumber(data);
		end
	else
		if errorMessage ~= nil then print(errorMessage) end;
		if errorReturn ~= nil then return errorReturn end;
	end
end


function dcl.GetTableSize(t)
	if type(t) ~= 'table' then return 0 end;
	local counter = 0;
	for k,v in pairs(t) do counter = counter + 1 end;
	return counter;
end


function dcl.MergeTables(t1, t2)
	for k, v in pairs(t2) do
		if (type(v) == "table") and (type(t1[k] or false) == "table") then
			dcl.MergeTables(t1[k], t2[k])
		else
			t1[k] = v
		end
	end
	return t1
end


function dcl.Dump(o)
    if type(o) == 'table' then
        local s = '{\n'
        for k,v in pairs(o) do
                if type(k) ~= 'number' then k = '"'..k..'"' end
                s = s .. '['..k..'] = ' .. dcl.Dump(v) .. '\n'
        end
        return s .. '}\n'
    else
        return ( tostring(o) )
    end
end


--This function returns a deep copy of a given table ---------------
function dcl.DeepCopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

-- converts a hex color to a valid RGB color
function dcl.ConvertHexColor(hexVal,alpha)

	hexVal = string.gsub(tostring(hexVal),"#","");
	hexVal = string.upper(hexVal);

	local sCheck = "0123456789ABCDEF";

	local iStart1 = string.find(sCheck,string.sub(hexVal,1,1)) - 1;
	local iStart2 = string.find(sCheck,string.sub(hexVal,2,2)) - 1;
	local iStart3 = string.find(sCheck,string.sub(hexVal,3,3)) - 1;
	local iStart4 = string.find(sCheck,string.sub(hexVal,4,4)) - 1;
	local iStart5 = string.find(sCheck,string.sub(hexVal,5,5)) - 1;
	local iStart6 = string.find(sCheck,string.sub(hexVal,6,6)) - 1;

	local RED = (1/255)*((iStart1 * 16) + iStart2);
	local GREEN = (1/255)*((iStart3 * 16) + iStart4);
	local BLUE = (1/255)*((iStart5 * 16) + iStart6);

	if alpha == nil then
		return RED,GREEN,BLUE;
	else
		return RED,GREEN,BLUE, alpha;
	end
end


function dcl.Round(num, idp)
	local mult = 10^(idp or 0);
    if num >= 0 then return math.floor(num * mult + 0.5) / mult;
    else return math.ceil(num * mult - 0.5) / mult end;
end


