
----------------------------------------------------------------------------------------------------------------------
--[[

RIFT MESSAGEBOX V1.0 BY GALUHAD

This class creates a Rift style message box.
It has two parameters, Message and Button.
Message is the message string you would like to display.
Button is the style of button you would like..

Button options:
		MBOK 		-- OK only (default if no option entered)
		MBOKCANCEL 	-- Ok / Cancel Buttons

The function returns the button pressed; either:
		MBOK		-- OK button pressed
		MBCANCEL	-- Cancel button pressed

You can use these values in your logic to determine how to process the players response.
E.G

tempMsg = MessageBox("Are you sure?",MBOKCANCEL);
tempMsg.ButtonClick = function (sender,args) -- Event executed when a button is pressed.. args returns a table with the value of the button pressed
	if args.Button == "MBYES" then
		--<do something here>
	end
end
--]]

msgBox = {};
msgBox.Back = dcl.ResourceDir.."MessageBox.png"; -- location where you stored the background image in your addon.

-----------------------------------------------------------------------------------------------------------------------

msgBox["Strings"] =
{
["MBOK"] = 		{[1] = "Ok";		[2] = "Ok";				[3] = "Ok";				[4] = "Ok";			[5] = "Ok";			[6] = "Ok";			[7] = "Ok";};
["MBCANCEL"] = 	{[1] = "Cancel";	[2] = "Rsilier";		[3] = "Kndigen";		[4] = "Cancel";		[5] = "Cancel";		[6] = "Cancel";		[7] = "Cancel";};
};


function msgBox.GetLanguage()
	local languages = {
		["English"] = 1;
		["French"] = 2;
		["German"] = 3;
		["Korean"] = 4;
		["Russian"] = 5;
		["Chinese"] = 6;
		["Taiwanese"] = 7;
		};

	local lang = Inspect.System.Language();
	if languages[lang] then return languages[lang] end;
	return 1;
end

msgBox.LangID = msgBox.GetLanguage();

-----------------------------------------------------------------------------------------------------------------------

_G["MBOK"] = "MBOK";
_G["MBCANCEL"] = "MBCANCEL";
_G["MBOKCANCEL"] = "MBOKCANCEL";

MessageBox = {};
_mtMessageBox = {};

function MessageBox.Constructor(SENDER,MESSAGE,BUTTON)

	if MESSAGE == nil then MESSAGE = "" end;

	local MESSAGEBOXWIDTH = 521;
	local MESSAGEBOXHEIGHT = 126;

	local BUTTONWIDTH = 170;
	local BUTTONTOP = MESSAGEBOXHEIGHT-40; -- -45

	local screenWidth = UIParent:GetWidth();
	local screenHeight = UIParent:GetHeight();

	local context = UI.CreateContext("msgBox_Context");

	local ScreenBlock = UI.CreateFrame("Frame", "msgBox_ScreenBlock", context)
	ScreenBlock:SetWidth(screenWidth);
	ScreenBlock:SetHeight(screenHeight);
	ScreenBlock:SetPoint("TOPLEFT",UIParent,"TOPLEFT");
	ScreenBlock:SetBackgroundColor(0, 0, 0, 0.2)
	ScreenBlock:SetVisible(true);

	local MessageWindow = UI.CreateFrame("Texture", "msgBox_MessageWindow", ScreenBlock)
	MessageWindow:SetWidth(MESSAGEBOXWIDTH);
	MessageWindow:SetHeight(MESSAGEBOXHEIGHT);
	MessageWindow:SetPoint("CENTER",ScreenBlock,"CENTER");
	MessageWindow:SetTexture(Inspect.Addon.Current(), msgBox.Back);
	MessageWindow:SetVisible(true);

	local MessageLabel = UI.CreateFrame("Text", "msgBox_MessageLabel", MessageWindow)
	MessageLabel:SetPoint("TOPLEFT",MessageWindow,"TOPLEFT",30,10);
	MessageLabel:SetWordwrap(true);
	MessageLabel:SetHeight(MESSAGEBOXHEIGHT-(MESSAGEBOXHEIGHT-BUTTONTOP)-15);
	MessageLabel:SetWidth(MESSAGEBOXWIDTH-60);
	MessageLabel:SetFontSize(16);
	MessageLabel:SetText(MESSAGE);


	if BUTTON == MBOKCANCEL then

		local btnOk = UI.CreateFrame("RiftButton", "Msg_btnOk", MessageWindow);
		btnOk:SetWidth(BUTTONWIDTH);
		btnOk:SetPoint("TOPLEFT",MessageWindow,"TOPLEFT",(MESSAGEBOXWIDTH/2)-BUTTONWIDTH,BUTTONTOP);
		btnOk:SetText(msgBox.Strings.MBOK[msgBox.LangID]);
		btnOk:SetLayer(100);

		btnOk.Event.LeftPress = function (Sender,Args)
			ScreenBlock.ButtonClick(sender,{["Button"]="MBOK";});
			ScreenBlock:SetVisible(false);
		end

		local btnCancel = UI.CreateFrame("RiftButton", "Msg_btnCancel", MessageWindow);
		btnCancel:SetWidth(BUTTONWIDTH);
		btnCancel:SetPoint("TOPLEFT",MessageWindow,"TOPLEFT",(MESSAGEBOXWIDTH/2),BUTTONTOP);
		btnCancel:SetText(msgBox.Strings.MBCANCEL[msgBox.LangID]);
		btnCancel:SetLayer(100);

		btnCancel.Event.LeftPress = function (Sender,Args)
			ScreenBlock.ButtonClick(sender,{["Button"]="MBCANCEL";});
			ScreenBlock:SetVisible(false);
		end

	else

		local btnOK = UI.CreateFrame("RiftButton", "Msg_btnOK", MessageWindow);
		btnOK:SetWidth(BUTTONWIDTH);
		btnOK:SetPoint("TOPLEFT",MessageWindow,"TOPLEFT",(MESSAGEBOXWIDTH/2)-(BUTTONWIDTH/2),BUTTONTOP);
		btnOK:SetText(msgBox.Strings.MBOK[msgBox.LangID]);
		btnOK:SetLayer(100);

		btnOK.Event.LeftPress = function (Sender,Args)
			ScreenBlock.ButtonClick(sender,{["Button"]="MBOK";});
			ScreenBlock:SetVisible(false);
		end

	end

	ScreenBlock.ButtonClick = function() end;

	return ScreenBlock;

end


function _mtMessageBox.__call(...)
    return MessageBox.Constructor(...);
end

setmetatable(MessageBox, _mtMessageBox);
